@@ -27,6 +27,15 @@
 
 #include "oscarcommon.h"
 
+static GHashTable *
+icq_get_account_text_table(PurpleAccount *account)
+{
+	GHashTable *table;
+	table = g_hash_table_new(g_str_hash, g_str_equal);
+	g_hash_table_insert(table, "login_label", (gpointer)_("ICQ UIN..."));
+	return table;
+}
+
 static PurplePluginProtocolInfo prpl_info =
 {
 	OPT_PROTO_MAIL_CHECK | OPT_PROTO_IM_IMAGE,
@@ -96,8 +105,10 @@ static PurplePluginProtocolInfo prpl_info =
 	NULL,					/* send_attention */
 	NULL,					/* get_attention_types */
 
-	/* padding */
-	NULL
+	sizeof(PurplePluginProtocolInfo),       /* struct_size */
+	icq_get_account_text_table, /* get_account_text_table */
+	NULL,					/* initiate_media */
+	NULL					/* can_do_media */
 };
 
 static PurplePluginInfo info =
